/*

            Sunclock for Windows

        Sun and Moon information panel

*/

#include "sunclock.h"

double moonphaseLast = -1E10;      /* Moon last phase calculation */
static double moonphaseInterval = CalculationInterval(300);
HICON micon = NULL;                /* Moon icon calculated for correct phase */

/*  ICONMOON  --  Construct icon for moon, given phase of moon.  */

static void iconmoon(HDC hDC, char *bmpname, int width, int CENTER,
                     int IRADIUS, int OFFSET,
                     int xpos, int ypos, double ph, int mm, int dd)
{
    int i, lx, rx, colour_mode;
    double cp, xscale, RADIUS = IRADIUS;

    HDC hMemoryDC;
    HBITMAP hColourmoon;
    unsigned char moon_template[] = {
        255,255,255,255,255,224,7,255,255,128,1,255,255,0,0,255,252,0,0,63,252,
        0,0,63,248,0,0,31,240,0,0,15,224,0,0,15,224,0,0,7,224,0,0,7,192,0,0,3,
        192,0,0,3,192,0,0,3,192,0,0,3,192,0,0,3,192,0,0,3,224,0,0,7,224,0,0,7,
        224,0,0,7,240,0,0,15,240,0,0,15,248,0,0,31,252,0,0,63,254,0,0,127,255,
        0,0,255,255,192,3,255,255,240,31,255,255,255,255,255,255,255,255,255,
        255,255,255,255,255,255,255,255
    };

    /* If the moon is new, draw an empty black square. */

    colour_mode = width == 64;
    if ((colour_mode == TRUE) && ((ph < 0.01) || (ph > 0.99))) {
        BitBlt(hDC, xpos, ypos, 58, 58, NULL, 0, 0, BLACKNESS);
        return;
    }

    hColourmoon = LoadBitmap(hInst, bmpname);

    hMemoryDC = CreateCompatibleDC(hDC);
    SelectObject(hMemoryDC, hColourmoon);

    /* If it's July 20th (in local time if we're running in real time,
       otherwise based on UTC), display the  Apollo  11  Commemorative
       Red  Dot at Tranquility Base.  Otherwise, just show the regular
       mare floor.  */

    if (mm != 7 || dd != 20) {
        if (colour_mode) {
            SetPixel(hMemoryDC, 41, 29, GetPixel(hMemoryDC, 42, 29));
        } else {
            SetPixel(hMemoryDC, 20, 14, GetPixel(hMemoryDC, 21, 14));
        }
    }

    xscale = cos(2 * PI * ph);
    for (i = 0; i < IRADIUS; i++) {
        cp = RADIUS * cos(asin((double) i / RADIUS));
        if (ph < 0.5) {
            rx = (int) (CENTER + cp);
            lx = (int) (CENTER + xscale * cp);
        } else {
            lx = (int) (CENTER - cp);
            rx = (int) (CENTER - xscale * cp);
        }

        /* We  now know the left and right  endpoints of the scan line
           for this y  coordinate.   We  raster-op  the  corresponding
           scanlines  from  the  source  pixrect  to  the  destination
           pixrect, offsetting to properly place it in the pixrect and
           reflecting vertically. */

        if (lx > 0) {
            BitBlt(hMemoryDC, 0, OFFSET + i, lx, 1, NULL, 0, 0, BLACKNESS);
            if ((rx + 1) < width) {
                BitBlt(hMemoryDC, rx + 1, OFFSET + i, width - rx, 1, NULL, 0, 0, BLACKNESS);
            }
            if (i != 0) {
                BitBlt(hMemoryDC, 0, OFFSET - i, lx, 1, NULL, 0, 0, BLACKNESS);
                if ((rx + 1) < width) {
                    BitBlt(hMemoryDC, rx + 1, OFFSET - i, width - rx, 1, NULL, 0, 0, BLACKNESS);
                }
            }
        }
    }

    if (colour_mode) {
        BitBlt(hDC, xpos, ypos, 58, 58, hMemoryDC, 2, 0, SRCCOPY);
    } else {
        BITMAP cmb;
        LPSTR mbitmap;
        HGLOBAL gbm;

        GetObject(hColourmoon, sizeof cmb, &cmb);

        gbm = GlobalAlloc(GMEM_MOVEABLE, ((DWORD) cmb.bmWidthBytes) * cmb.bmHeight * cmb.bmPlanes);
        if (gbm != NULL) {
            mbitmap = GlobalLock(gbm);
            GetBitmapBits(hColourmoon, ((long) cmb.bmWidthBytes) * cmb.bmHeight * cmb.bmPlanes, mbitmap);
            if (micon != NULL) {
                DestroyIcon(micon);
                micon = NULL;
            }
            micon = CreateIcon(hInst, cmb.bmWidth, cmb.bmHeight, (BYTE) cmb.bmPlanes,
                                      (BYTE) cmb.bmBitsPixel, moon_template, mbitmap);
            GlobalUnlock(gbm);
            GlobalFree(gbm);
        }
    }
    DeleteDC(hMemoryDC);
    DeleteObject(hColourmoon);
} 

//	UPDMOONICON  --  Update Moon icon if required.

void updMoonIcon(HDC hDC, double jd)
{
    if (NeedToCalculate(moonphase, jd)) {
	    double p, aom, cphase, cdist, cangdia, csund, csuang;
	    
        p = phase(jd, &cphase, &aom, &cdist, &cangdia, &csund, &csuang);
        iconmoon(hDC, "moonicon", 32, /* X ctr */ 16, /* X radius */ 14,
                                      /* Y centre */ 14, 0, 0, p, 0, 0);
    }
}

